import fetch from 'node-fetch';

// Function to handle form submission and interact with Listmonk
export async function submitToListmonk(req, res) {
    const { name, email, message } = req.body;

    const listmonkUrl = process.env.LISTMONK_URL;
    const listmonkUsername = process.env.LISTMONK_USERNAME;
    const listmonkPassword = process.env.LISTMONK_PASSWORD;
    const listmonkBaseUrl = process.env.LISTMONK_BASE_URL;
    const listId = parseInt(process.env.LIST_ID, 10);

    // Encode username and password as base64
    const auth = Buffer.from(`${listmonkUsername}:${listmonkPassword}`).toString('base64');

    console.log(`Sending request to Listmonk with URL: ${listmonkUrl}`);

    const payload = {
        email,
        name,
        lists: [listId],
        status: "enabled",
        fields: { message }
    };

    console.log('Sending request to Listmonk with payload:', payload);

    async function fetchWithRetry(url, options, retries = 3) {
        for (let i = 0; i < retries; i++) {
          console.log(`Attempt ${i + 1} sending request to ${url}`);
          try {
            const res = await fetch(url, options);
            if (!res.ok) throw new Error(`HTTP ${res.status}`);
            return res;
          } catch (err) {
            console.log(`Attempt ${i + 1} failed: ${err}`);
            if (i === retries - 1) throw err;
            await new Promise(resolve => setTimeout(resolve, 500));
          }
        }
      }
      
    try {
        // Add subscriber
        const response = await fetchWithRetry(listmonkUrl, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'Authorization': `Basic ${auth}`
            },
            body: JSON.stringify(payload),
        },12);

        if (!response.ok) {
            const errorBody = await response.text();
            throw new Error(`Listmonk API error: ${response.statusText}. Response body: ${errorBody}`);
        }

        const result = await response.json();
        console.log('Subscriber added successfully:', result);

        // Create a new campaign for the subscriber
        const createCampaign = async (subject, body) => {
            const createCampaignUrl = `${listmonkBaseUrl}/campaigns`;
            const campaignPayload = {
                name: `Welcome email for ${email}`,
                subject: subject,
                from_email: "Janek <janek@melonion.me>",
                body: body,
                type: "regular",
                to_emails: [email],
                lists: [listId],
                content_type: "richtext",
                status: "scheduled", 
                schedule: new Date().toISOString() 
            };

            const response = await fetch(createCampaignUrl, {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                    'Authorization': `Basic ${auth}`,
                },
                body: JSON.stringify(campaignPayload),
            });

            if (!response.ok) {
                const errorBody = await response.text();
                throw new Error(`Listmonk API error (Create Campaign): ${response.statusText}. Response body: ${errorBody}`);
            }

            const result = await response.json();
            return result.data.id;
        };

        // Campaign subject and body content
        const subject = "👋 Welcome to the Progodyssey Family!";
        const body = `
        <h1>👋 Welcome to the Progodyssey Family, {{ .Subscriber.Name }}!</h1>

        <p>We’re thrilled to have you as part of the Progodyssey Traineeship journey! 🎉
        This program is designed to equip you with the skills, experience, and network to thrive in the tech world and beyond.</p>

        <p>Here’s what you can expect in the coming weeks:</p>
        <ul>
            <li>1️⃣ <strong>Learning Opportunities:</strong> Access to tutorials, and hands-on projects to sharpen your skills.</li>
            <li>2️⃣ <strong>Mentorship:</strong> Personal Guidance to help you navigate your journey.</li>
            <li>3️⃣ <strong>Community:</strong> A supportive group of peers and mentors working together towards success.</li>
        </ul>

        <p>To kick things off, go through the Progodyssey handbook, pick a course and submit your reflection essay to <a href="mailto:reflection@progodyssey.com">reflection@progodyssey.com</a>.</p>
        <p>Janek’s Blog could also have some insight for you.</p>

        <p><em>Encouragement:</em><br>
        <em>"Success is the sum of small efforts repeated day in and day out."</em> – <cite>Robert Collier</cite></p>

        <p>Let’s take the first step together!<br>
        If you have any questions or need assistance, feel free to reply to this email or contact us at <a href="mailto:hello@progodyssey.com">hello@progodyssey.com</a></p>

        <p>Looking forward to seeing you thrive,<br>
        <strong>The Progodyssey Team</strong></p>
        `;

        // 1. Create campaign
        const newCampaignId = await createCampaign(subject, body);
        console.log(`Created new campaign with ID: ${newCampaignId}`);

        // 2. Dispatch campaign
        const dispatchUrl = `${listmonkBaseUrl}/campaigns/${newCampaignId}/status`;
        const dispatchPayload = {
          status: 'running'
        };

        const dispatchResponse = await fetch(dispatchUrl, {
          method: 'PUT',
          headers: {
            'Content-Type': 'application/json',
            'Authorization': `Basic ${auth}`,
          },
          body: JSON.stringify(dispatchPayload),
        });

        if (!dispatchResponse.ok) {
          const errorBody = await dispatchResponse.text();
          console.error('Response from Listmonk (Dispatch):', errorBody);
          throw new Error(`Listmonk API error (Dispatch Campaign): ${dispatchResponse.statusText}. Response body: ${errorBody}`);
        }

        const dispatchResult = await dispatchResponse.json();
        console.log('Campaign dispatched successfully:', dispatchResult);

        res.status(200).json({ message: 'Subscription and welcome email sent successfully!', result });

    } catch (error) {
        console.error('Error submitting to Listmonk:', error);
        res.status(500).json({ message: 'Failed to submit to Listmonk', error: error.message });
    }
}
